
# Written by Nirmal Patel (http://nirmalpatel.com)
# Version 1.1

import os, urllib, urllib2, datetime, logging
from base64 import encodestring


NOTIFO_NOTIFICATION_URL = "https://api.notifo.com/v1/send_notification"

GROWL_APP_LABEL = "Growl"
GROWL_USER_IDLE_SUBJECT = "User went idle"
GROWL_USER_RETURNED_SUBJECT = "User returned"

SCRIPT_DIR = os.path.expanduser("~/Library/Application Support/Growl/")
LOG_PATH = os.path.expanduser("~/Library/Logs/GrowlToNotifo.log")
IDLE_FILE = "userIdle.lock"

ABBREVIATED_MONTH_NAMES = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", 
                           "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

def sendNotification(msg, label, title):
    
    data = {"to"    : NOTIFO_USERNAME,
            "msg"   : msg,
            "label" : label,
            "title" : title}
    data = urllib.urlencode(data)

    auth = encodestring('%s:%s' % (NOTIFO_USERNAME, NOTIFO_API_SECRET)).replace('\n', '')
    request = urllib2.Request(NOTIFO_NOTIFICATION_URL, data)
    request.add_header("Authorization", "Basic %s" % auth)
    try:
        response = urllib2.urlopen(request)
    except IOError: # no connection
        return None
    
    return response.read()



def parseGrowlLogEntry(entry):
    try:
        app, rest = entry.strip().split(":", 1)
        app = app.strip()
        subject, rest = rest.strip().split("(", 1)
        subject = subject.strip()
        message, priority = rest.strip().rsplit(")", 1)
        message = message.strip()
        priority = int(priority.rsplit(" ", 1)[-1])
    
        return app, subject, message, priority
    except:
        logging.error("Unable to parse: " + entry)
        return [None] * 4
    
    
def logEntriesToList():

    growlLogFile = open(LOG_PATH)
    growlEntries = growlLogFile.readlines()
    growlLogFile.close()

    entries = []
    for line in growlEntries:
        entryStart = False
        try:
            #Jun 24, 2010 [Thu,
            # Was using the whole datetime string "Jun 24, 2010 [Thu, 24], 10:24"
            # but strptime did not allow me to place %d for date twice in the format string
            datePortion = " ".join(line.split(" ", 4)[:-1])
            datetime.datetime.strptime(datePortion, "%b %d, %Y [%a,")
            entryStart = True
        except ValueError:
            #Not a date so this is just another line of the previous notification
            pass
    
        if (entryStart):
            entries.append(line)
        else:
            entries[-1] += line

    return entries


def checkIfIdleAlready():
    os.chdir(SCRIPT_DIR)

    #Check to see if user was already idle or is idle now
    idleFromLastRun = os.access(IDLE_FILE, os.F_OK)
    idle = idleFromLastRun

    return idle

def processEntries(entries, idle):

    for entry in entries:
        entry = entry.strip()
        data = entry.split(" ", 6)
        rest = data[-1].strip()
        if ((":" in rest) and ("(" in rest)):

            app, subject, message, priority = parseGrowlLogEntry(rest)
        
            if (not app):
                continue
        
            if (app == GROWL_APP_LABEL):
                if (subject == GROWL_USER_IDLE_SUBJECT):
                    idle = True
                    #creating idle lock file to save info between runs
                    open(IDLE_FILE, "w").close()
                elif (subject == GROWL_USER_RETURNED_SUBJECT):
                    idle = False
                    #delete idle lock file 
                    os.remove(IDLE_FILE)
                    

            else:
                if (idle):
                    sendNotification(message, app, subject)
                else:
                    #Don't send
                    pass

def clearLogFile():
    #clear log file
    open(LOG_PATH, "w").close()    


def main():
    entries = logEntriesToList()
    idle    = checkIfIdleAlready()
    processEntries(entries, idle)
    clearLogFile()
    

if __name__ == "__main__":
    main()